@@ -8,12 +8,10 @@ Run PACKAGE.x jobs.
 """
 
 
-from ase import constraints, io
-from ase.calculators.calculator import FileIOCalculator, CalculationFailed, equal, compare_atoms
-from ase.io.formats import F
-from numpy.core.fromnumeric import sort
-import pseudo
+from ase import io
+from ase.calculators.calculator import FileIOCalculator, CalculationFailed, equal, compare_atoms, PropertyNotPresent
 from xespresso.xio import write_espresso_in, read_espresso_input, read_espresso_asei, write_espresso_asei, get_atomic_species, get_atomic_constraints
+from ase.io.espresso import read_espresso_in
 import os
 import shutil
 import numpy as np
@@ -30,10 +28,9 @@ logging.basicConfig(stream=sys.stdout,
                             '[%(funcName)-20s]: %(message)s'),
                     level=logging.INFO)
 
-logger = logging.getLogger('xespresso')
+logger = logging.getLogger(__name__)
+
 
-config_files = [os.path.join(os.environ['HOME'], '.xespressorc'),
-            '.xespressorc']
 error_template = 'Property "%s" not available. Please try running Quantum\n' \
                  'Espresso first by calling Atoms.get_potential_energy().'
 
@@ -84,14 +81,14 @@ class Espresso(FileIOCalculator):
               >>> calc.nscf_calculate()
         """
         print("{0:=^60}".format(package))
-        self.logger = logger
         if debug:
-            self.logger.setLevel(debug)
+            logger.setLevel(debug)
         self.set_label(label, prefix)
         self.scf_directory = None
         self.scf_parameters = None
         self.scf_results = None
-        kwargs = self.check_input(kwargs)
+        kwargs = self.check_input(kwargs, prefix = self.prefix)
+        self.ase_parameters = kwargs
         FileIOCalculator.__init__(self, restart = self.directory, 
                                   label = self.label, atoms = atoms, **kwargs)
         if atoms:
@@ -120,21 +117,24 @@ class Espresso(FileIOCalculator):
         self.asei = os.path.join(self.directory, '%s.asei'%self.prefix)
         self.asei_temp = os.path.join(self.directory, '.%s.asei_temp'%self.prefix)
         self.save_directory = os.path.join(self.directory, '%s.save'%self.prefix)
-        self.logger.debug('Directory: %s'%(self.directory))
-        self.logger.debug('Prefix: %s'%(self.prefix))
+        logger.debug('Directory: %s'%(self.directory))
+        logger.debug('Prefix: %s'%(self.prefix))
+
     def check_pseudopotentials(self, pseudopotentials):
         """
         """
         # pseudopotentials
-        if 'species' not in self.atoms.info:
+        if 'species' not in self.atoms.arrays:
             all_species = set(self.atoms.get_chemical_symbols())
         else:
-            all_species = set(self.atoms.info['species'])
+            all_species = set(self.atoms.arrays['species'])
         new_pseudopotentials = {}
         for species in all_species:
             assert species in pseudopotentials, '\n  Species: %s, does not have a pseudopotentials!'%(species)
             new_pseudopotentials[species] = pseudopotentials[species]
-    def check_input(self, kwargs, package = 'PW'):
+
+    @classmethod    
+    def check_input(cls, kwargs, prefix = 'pw', package = 'PW'):
         """
         Chekc input paramter type.
         set parameters for pw.x
@@ -144,72 +144,12 @@ class Espresso(FileIOCalculator):
         sorted_parameters, unuse_parameters = sort_qe_input(kwargs)
         check_qe_input(sorted_parameters['input_data'])
         if unuse_parameters:
-            self.logger.debug('Warnning: parameter %s not used, please check.'%unuse_parameters)
-        sorted_parameters['input_data']['CONTROL']['prefix'] = self.prefix
+            logger.debug('Warnning: parameter %s not used, please check.'%unuse_parameters)
+        sorted_parameters['input_data']['CONTROL']['prefix'] = prefix
         sorted_parameters['input_data']['CONTROL']['verbosity'] = 'high'
-        ase_parameters = sorted_parameters
-        self.ase_parameters = ase_parameters
         # pprint(sorted_parameters)
         return sorted_parameters
-    def set_queue(self, package = None, parallel = None, queue = None, command = None):
-        '''
-        If queue, change command to "sbatch .job_file".
-        The queue information are written into '.job_file'
-        '''
-        if queue is None:
-            queue = self.queue
-        else:
-            self.queue = queue
-        if package is None:
-            package = self.package
-        else:
-            self.package = package
-        if parallel is None:
-            parallel = self.parallel
-        else:
-            self.parallel = parallel
-        if command is None:
-            command = os.environ.get('ASE_ESPRESSO_COMMAND')
-        if 'PACKAGE' in command:
-            if 'pw' in package:
-                command = command.replace('PACKAGE', package, 1)
-                command = command.replace('PACKAGE', 'pw', 2)
-            else:
-                command = command.replace('PACKAGE', package)
-        if 'PREFIX' in command:
-            command = command.replace('PREFIX', self.prefix)
-        if 'PARALLEL' in command:
-            command = command.replace('PARALLEL', parallel)
-        self.logger.debug('Espresso command: %s'%(command))
-        if queue:
-            script = ''
-            if 'config' in queue:
-                cf = os.path.join(os.environ['HOME'], queue['config'])
-                file = open(cf, 'r')
-                script = file.read()
-                # del queue['config']
-            else:
-                for cf in config_files:
-                    if os.path.exists(cf):
-                        file = open(cf, 'r')
-                        script = file.read()
-            jobname = self.prefix
-            with open('%s/.job_file' % self.directory, 'w') as fh:
-                fh.writelines("#!/bin/bash\n")
-                fh.writelines("#SBATCH --job-name=%s \n" % jobname)
-                fh.writelines("#SBATCH --output=%s.out\n" % self.prefix)
-                fh.writelines("#SBATCH --error=%s.err\n" % self.prefix)
-                fh.writelines("#SBATCH --wait\n")
-                for key, value in queue.items():
-                    if key == 'config': continue
-                    if value:
-                        fh.writelines("#SBATCH --%s=%s\n" %(key, value))
-                fh.writelines("%s \n"%script)
-                fh.writelines("%s \n" % command)
-            self.command = "sbatch {0}".format('.job_file')
-        else:
-            self.command = command
-        self.logger.debug('Queue command: %s'%(self.command))
+
     def read(self, label):
         """Read the files in a calculation if they exist.
         This function reads self.parameters and atoms.
@@ -220,7 +160,7 @@ class Espresso(FileIOCalculator):
         self.restart_atoms = None
         self.restart_parameters = {}
         if os.path.exists(label) and os.path.exists(self.asei):
-            self.logger.debug('Try to read information from folder: %s.'%(label))
+            logger.debug('Try to read information from folder: %s.'%(label))
             try:
                 # atoms, parameters = self.read_xml_file()
                 atoms, parameters = read_espresso_asei(self.asei)
@@ -229,9 +169,9 @@ class Espresso(FileIOCalculator):
                 # self.parameters = copy.deepcopy(parameters)
                 self.read_results()
             except Exception as e:
-                self.logger.debug('Directory: %s exist, faild to read. %s'%(label, e))
+                logger.debug('Directory: %s exist, faild to read. %s'%(label, e))
         else:
-            self.logger.debug('Directory %s is not a espresso folder, start a new calculation:'%(label))
+            logger.debug('Directory %s is not a espresso folder, start a new calculation:'%(label))
     def set(self, **kwargs):
         """
         """
@@ -252,19 +192,23 @@ class Espresso(FileIOCalculator):
             for i in range(len(atoms)):
                 if compare_atoms(self.restart_atoms[i], atoms[i], tol=tol,
                              excluded_properties=set(self.ignored_changes)):
-                    self.logger.debug('Atoms changed, run a new calculation')
+                    logger.debug('Atoms changed, run a new calculation')
                     return True
         else:
             if compare_atoms(self.restart_atoms, atoms, tol=tol,
                              excluded_properties=set(self.ignored_changes)):
-                self.logger.debug('Atoms changed, run a new calculation')
+                logger.debug('Atoms changed, run a new calculation')
                 return True
         # if self.igonre_parameters:
-            # self.logger.debug('Parameter: %s are ignored, results are not affected.'%self.igonre_parameters)
+            # logger.debug('Parameter: %s are ignored, results are not affected.'%self.igonre_parameters)
         if self.changed_parameters:
-            self.logger.debug('Parameter: %s changed, results are affected'%self.changed_parameters)
+            logger.debug('Parameter: %s changed, results are affected'%self.changed_parameters)
+            return True
+        logger.debug('Same geometry and parameters, use previous results.')
+        converged, meg = self.read_convergence()
+        if converged > 0:
+            logger.debug('Not converged: %s'%meg)
             return True
-        self.logger.debug('Same geometry and parameters, use previous results.')
         return False
     def compare_parameters(self, para1, para2, ignore = []):
         """
@@ -344,26 +288,33 @@ class Espresso(FileIOCalculator):
         get atomic species
         '''
         pwo = os.path.join(self.directory, '%s.pwo'%self.prefix)
+        pwi = os.path.join(self.directory, '%s.pwi'%self.prefix)
         convergence, meg = self.read_convergence()
-        if convergence == 0:
-            try:
-                output = io.read(pwo)
-                atomic_species = get_atomic_species(pwo)
-                constraints = get_atomic_constraints(pwo, len(output))
-                output.set_constraint(None)
-                output.set_constraint(constraints)
-                if atomic_species: output.info['species'] = atomic_species
-                self.calc = output.calc
-                self.results = output.calc.results
-                self.results['atoms'] = output
-                self.efermi = self.get_fermi_level()
-                # self.nspins = self.get_number_of_spins()
-                self.logger.debug('Read result successfully!')
-            except Exception as e:
-                self.logger.debug('Read output: %s, failed! %s'%(pwo, e))
-        else:
-            self.logger.debug('Not converged. %s'%(meg))
-            # self.logger.debug('Read result failed!')
+        if convergence != 0:
+            logger.debug('Not converged. %s'%(meg))
+        try:
+            atoms = read_espresso_in(pwi)
+            self.results['atoms'] = atoms
+        except Exception as e:
+            logger.debug('Read input: %s, failed! %s'%(pwi, e))
+        try:
+            output = io.read(pwo)
+            atomic_species = get_atomic_species(pwo)
+            constraints = get_atomic_constraints(pwo, len(output))
+            output.set_constraint(None)
+            output.set_constraint(constraints)
+            if atomic_species:
+                output.new_array('species', np.array(atomic_species, dtype = 'U20'))
+            self.calc = output.calc
+            self.results = output.calc.results
+            self.results['atoms'] = output
+            self.efermi = self.get_fermi_level()
+            # self.nspins = self.get_number_of_spins()
+            logger.debug('Read result successfully!')
+        except Exception as e:
+            logger.debug('Read output: %s, failed! %s'%(pwo, e))
+        self.results['convergence'] = convergence
+        # logger.debug('Read result failed!')
         # pwos = [file for file in os.listdir(self.directory) if pwo in file]
         # output = None
         # for pwo in pwos:
@@ -372,11 +323,13 @@ class Espresso(FileIOCalculator):
             # atomic_species = get_atomic_species(pwo)
             
     def write_input(self, atoms, properties=None, system_changes=None):
+        from xespresso.scheduler import set_queue
         FileIOCalculator.write_input(self, atoms, properties, system_changes)
-        self.set_queue()
+        set_queue(self)
         write_espresso_in(self.label + '.pwi', atoms, **self.parameters)
         write_espresso_asei(self.label + '.asei', atoms, self.parameters)
-        self.logger.debug('Write input successfully')
+        logger.debug('Write input successfully')
+    
     def check_xml_file(self):
         '''
         If data-file-schema.xml is readable, then xml_end = True
@@ -404,16 +357,16 @@ class Espresso(FileIOCalculator):
     def read_xml_file(self):
         from xespresso.xml_parser import xml_parser
         if not self.check_xml_file():
-            self.logger.debug('xml file not finished.')
+            logger.debug('xml file not finished.')
         #
         xmlfile = os.path.join(self.save_directory, 'data-file-schema.xml')
         if not os.path.exists(xmlfile):
             return None, {}
         try:
             atoms, xml_input = xml_parser(xmlfile)
-            self.logger.debug('Read geometry and parameters from xml file successfully.')
+            logger.debug('Read geometry and parameters from xml file successfully.')
         except:
-            self.logger.debug('Read xml file failed.')
+            logger.debug('Read xml file failed.')
             return None, {}
         return atoms, xml_input
     def read_convergence(self):
@@ -433,7 +386,7 @@ class Espresso(FileIOCalculator):
         # if self.queue:
         #     errfile = self.label + '.err'
         #     if not os.path.exists(errfile):
-        #         self.logger.debug('%s not exists'%errfile)
+        #         logger.debug('%s not exists'%errfile)
         #         # return 2, 'Pending or not submit'
         #     else:
         #         errs = ['RESTART', 'pw.x', 'out-of-memory', 
@@ -447,12 +400,12 @@ class Espresso(FileIOCalculator):
         #                     cancelled = True
         #                 for err in errs:
         #                     if err in line:
-        #                         self.logger.debug('Need restart')
+        #                         logger.debug('Need restart')
         #                         return 3, line
         #             # cancelled by owner
         #             if len(lines) > 0:
         #                 if cancelled:
-        #                     self.logger.debug('Cancelled')
+        #                     logger.debug('Cancelled')
         #                     return -1, lines[0]
         # no error from queue, or job not run in the queue
         output = self.label + '.pwo'
@@ -468,41 +421,20 @@ class Espresso(FileIOCalculator):
             lastlines = lines[-n:-1]
             for line in lastlines:
                 if line.rfind('too many bands are not converged') > -1:
-                    self.logger.debug('Need restart')
+                    logger.debug('Need restart')
                     return 1, 'Reason: %s'%(line)
                 if line.rfind('convergence NOT achieved after') > -1:
-                    self.logger.debug('Need restart: %s'%line)
+                    logger.debug('Need restart: %s'%line)
                     return 1, 'Reason: %s'%(line)
                 if line.rfind('Maximum CPU time exceeded') > -1:
-                    self.logger.debug('Need restart')
+                    logger.debug('Need restart')
                     return 2, 'Reason: %s'%(line)
                 if line.rfind('JOB DONE.') > -1:
-                    self.logger.debug('JOB DONE')
+                    logger.debug('JOB DONE')
                     return 0, line
-            self.logger.debug('Not converged, %s'%lastlines[-1])
+            logger.debug('Not converged, %s'%lastlines[-1])
         return 4, line
-    def read_convergence_post(self, package = 'pw'):
-        '''
-        Read the status of the calculation.
-        {
-        '0': 'Done',
-        }
-        '''
-        
-        output = self.label + '.%so'%package
-        if not os.path.exists(output):
-            # print('%s not exists'%output)
-            return False, 'No pwo output file'
-        with open(output, 'r') as f:
-            lines = f.readlines()
-            if len(lines) == 0: return False, 'pwo file has nothing'
-            nlines = len(lines)
-            n = min([100, nlines])
-            for line in lines[-n:-1]:
-                if line.rfind('JOB DONE.') > -1:
-                    self.logger.debug('JOB DONE.')
-                    return True, line
-        return False, line
+    
     def run(self, atoms = None, restart = False):
         '''
         run and restart
@@ -515,11 +447,11 @@ class Espresso(FileIOCalculator):
         self.atoms.calc = self
         if not restart:
             try:
-                self.logger.debug('Run step: %s'%icount)
+                logger.debug('Run step: %s'%icount)
                 self.atoms.get_potential_energy()
                 converged, meg0 = self.read_convergence()
             except Exception as e:
-                self.logger.debug('Not converge: %s'%e)
+                logger.debug('Not converge: %s'%e)
                 converged, meg0 = self.read_convergence()
                 icount += 1
                 if converged == 0:
@@ -529,13 +461,13 @@ class Espresso(FileIOCalculator):
         exitfile = os.path.join(self.directory, 'EXIT')
         while converged > 0 or restart == 2:
             if os.path.exists(exitfile):
-                self.logger.debug('Exit file exists!')
+                logger.debug('Exit file exists!')
                 os.remove(exitfile)
                 sys.exit()
             restart = 1
             self.backup_file('%s.pwo'%self.prefix, directory = self.directory)
             self.backup_file('data-file-schema.xml', directory = self.save_directory)
-            self.logger.debug('Run step: %s'%icount)
+            logger.debug('Run step: %s'%icount)
             xml_end = self.check_xml_file()
             if xml_end:
                 self.parameters['input_data']['CONTROL']['restart_mode'] = 'restart'
@@ -550,25 +482,25 @@ class Espresso(FileIOCalculator):
                     mixing_beta = float(self.parameters['input_data']['ELECTRONS']['mixing_beta'])
                 else:
                     mixing_beta = 0.7
-                if electron_maxstep < 200:
+                if electron_maxstep < 100:
                     self.parameters['input_data']['ELECTRONS']['electron_maxstep'] = electron_maxstep + 50
-                    self.logger.debug('electron_maxstep change from %s to %s'%(electron_maxstep, electron_maxstep + 50))
+                    logger.debug('electron_maxstep change from %s to %s'%(electron_maxstep, electron_maxstep + 50))
                 self.parameters['input_data']['ELECTRONS']['mixing_beta'] = mixing_beta/2.0
-                self.logger.debug('mixing_beta change from %s to %s'%(mixing_beta, mixing_beta/2.0))
+                logger.debug('mixing_beta change from %s to %s'%(mixing_beta, mixing_beta/2.0))
             try:
                 icount += 1
                 atoms.get_potential_energy()
                 converged, meg = self.read_convergence()
             except Exception as e:
-                self.logger.debug('Not converge: %s'%e)
+                logger.debug('Not converge: %s'%e)
                 converged, meg = self.read_convergence()
-                # meg = e
+                meg = str(e)
                 restart = 2
-                if converged == 0:
-                    converged = 4
-            self.logger.debug('%s, %s'%(converged, meg[0:-1]))
-            if meg == meg0:
-                self.logger.debug('Exit! Maybe deadblock! Same error message! %s'%meg)
+                # if converged == 0:
+                converged = 4
+            logger.debug('%s, %s'%(converged, meg))
+            if meg == meg0 and converged != 2:
+                logger.debug('Exit! Maybe deadblock! Same error message! %s'%meg)
                 sys.exit()
             meg0 = meg
     def backup_file(self, src, directory = '.'):
@@ -593,207 +525,7 @@ class Espresso(FileIOCalculator):
         if flag:
             shutil.copy(src, new_src)
         return 0
-    def nscf(self, calculation = 'nscf', package = 'pw', queue = False, parallel = '', kpts = (10, 10, 10), **kwargs):
-        import copy
-        # save scf parameters
-        # save scf parameters
-        print("{0:=^60}".format(calculation))
-        if not self.scf_directory:
-            self.scf_directory = self.directory
-        if not self.scf_parameters:
-            self.scf_parameters = copy.deepcopy(self.parameters)
-        if not self.scf_results:
-            self.scf_results = copy.deepcopy(self.results)
-            self.efermi = self.get_fermi_level()
-        # read new atoms from results and set magmoms
-        self.atoms = self.results['atoms']
-        self.parameters = copy.deepcopy(self.scf_parameters)
-        self.parameters['kpts'] = kpts
-        self.parameters['input_data']['CONTROL']['calculation'] = calculation
-        self.parameters['input_data']['CONTROL']['verbosity'] = 'high'
-        self.parameters['input_data']['CONTROL']['outdir'] = '../'
-        self.parameters['input_data']['SYSTEM']['occupations'] = 'tetrahedra'
-        self.parameters['input_data']['SYSTEM'].pop('degauss', None)
-        # self.parameters['restart_mode'] = 'restart'
-        for key, value in kwargs.items():
-            self.parameters[key] = value
-        self.parameters = self.check_input(self.parameters)
-        # create working directory
-        self.directory = os.path.join(self.scf_directory, '%s/'%calculation)
-        self.label = os.path.join(self.directory, self.prefix)
-        self.nscf_asei = os.path.join(self.directory, '%s.nscf_asei'%self.prefix)
-        # create working directory
-        self.directory = os.path.join(self.scf_directory, '%s/'%calculation)
-        self.label = os.path.join(self.directory, self.prefix)
-        self.set_label(self.directory, self.prefix)
-        self.set_queue(package = package, parallel = parallel, queue = queue)
-    def nscf_calculate(self, ):
-        from xespresso.utils import get_hash
-        # read information of the charge-density file
-        chargeFile = os.path.join(self.scf_directory, '%s.save/charge-density.dat'%self.prefix)
-        self.state_info = get_hash(chargeFile)
-        self.state_parameters = self.parameters
-        output, meg = self.read_convergence_post('pw')
-        if output:
-            if os.path.isfile(self.nscf_asei):
-                system_changes =  self.check_state_post(self.nscf_asei, package='PW')
-                if not system_changes:
-                    self.logger.debug('Use previous results!')
-                    return 0
-        write_espresso_asei(self.nscf_asei, self.state_info, self.state_parameters)
-        self.calculate()
-        # self.state_info = get_hash(chargeFile)
-        # write_espresso_asei(self.nscf_asei, self.state_info, self.state_parameters)
-        # self.read_results()
-
-    
-    def post(self, queue = False, package = 'dos', parallel = '', **kwargs):
-        '''
-        todo: 
-        '''
-        from xespresso.utils import get_hash
-        print('{0:=^60}'.format(package))
-        self.directory = os.path.join(self.scf_directory, '%s/'%package)
-        self.set_label(self.directory, self.prefix)
-        self.post_asei = os.path.join(self.directory, '%s.post_asei'%self.prefix)
-        kwargs['prefix'] = self.prefix
-        kwargs['outdir'] = '../'
-        self.state_parameters = copy.deepcopy(kwargs)
-        for wfc in ['wfc1', 'wfcdw1']:
-            wfcFile = os.path.join(self.scf_directory, '%s.save/%s.dat'%(self.prefix, wfc))
-            if os.path.isfile(wfcFile):
-                self.state_info = get_hash(wfcFile)
-        output, meg = self.read_convergence_post(package)
-        if output:
-            self.logger.debug('Previous calculation done.')
-            if os.path.isfile(self.post_asei):
-                system_changes =  self.check_state_post(self.post_asei, package)
-                if not system_changes:
-                    self.logger.debug('File and Parameters did not change. Use previous results!')
-                    return 0
-        self.post_write_input(package, **self.state_parameters)
-        write_espresso_asei(self.post_asei, self.state_info, self.state_parameters)
-        self.set_queue(package = package, parallel = parallel, queue = queue)
-        self.post_calculate()
-        self.post_read_results()
-    def check_state_post(self, asei, package):
-        old_state_info, old_state_parameters = read_espresso_asei(asei, package)
-        if not self.state_info == old_state_info:
-            self.logger.debug('File in save changed')
-            return True
-        elif not self.state_parameters == old_state_parameters:
-            self.logger.debug('Parameters changed')
-            return True
-        else:
-            return False
-    def post_write_input(self, package, **kwargs):
-        state_parameters = {
-        'dos':  {'DOS': ['prefix', 'outdir', 'bz_sum', 'ngauss', 'degauss', 
-                        'Emin', 'Emax', 'DeltaE', 'fildo', ]
-                        },
-        'pp':   {'INPUTPP': ['prefix', 'outdir', 'filplot', 'plot_num', 
-                       'spin_component', 'spin_component', 'emin', 'emax', 
-                       'delta_e', 'degauss_ldos', 'sample_bias', 'kpoint', 
-                       'kband', 'lsign', 'spin_component', 'emin', 'emax', 
-                       'spin_component', 'spin_component', 'spin_component', 
-                       'spin_component'], 
-                'PLOT': ['nfile', 'filepp', 'weight', 'iflag', 'output_format', 
-                    'fileout', 'interpolation', 'e1', 'x0', 'nx', 'e1', 'e2', 
-                    'x0', 'nx', 'ny', 'e1', 'e2', 'e3', 'x0', 'nx', 'ny', 'nz', 
-                    'radius', 'nx', 'ny']
-                    },
-        'projwfc': {'PROJWFC': ['prefix', 'outdir', 'ngauss', 'degauss', 
-                    'Emin', 'Emax', 'DeltaE', 'lsym', 'pawproj', 'filpdos', 'filproj', 
-                    'lwrite_overlaps', 'lbinary_data', 'kresolveddos', 'tdosinboxes', 
-                    'n_proj_boxes', 'irmin(3,n_proj_boxes)', 'irmax(3,n_proj_boxes)', 
-                    'plotboxes', ]
-                    },
-        'hp': {'INPUTHP': ['prefix','outdir','iverbosity','max_seconds','nq1','nq2',
-                           'nq3','skip_equivalence_q','determine_num_pert_only',
-                           'find_atpert','docc_thr','skip_type','equiv_type',
-                           'perturb_only_atom','start_q','last_q','sum_pertq',
-                           'compute_hp','conv_thr_chi','thresh_init','ethr_nscf',
-                           'niter_max','alpha_mix(i)','nmix','num_neigh','lmin','rmax',]},
-        'bands': {'BANDS': ['prefix', 'outdir', 'filband', 'spin_component', 'lsigma', 
-                            'lp', 'filp', 'lsym', 'no_overlap', 'plot_2d', 'firstk', 'lastk']
-                            },
-        'ph': {'INPUTPH': ['amass', 'outdir', 'prefix', 'niter_ph', 'tr2_ph', 
-                           'alpha_mix(niter)', 'nmix_ph', 'verbosity', 'reduce_io', 
-                           'max_seconds', 'fildyn', 'fildrho', 'fildvscf', 'epsil', 
-                           'lrpa', 'lnoloc', 'trans', 'lraman', 'eth_rps', 'eth_ns', 
-                           'dek', 'recover', 'low_directory_check', 'only_init', 'qplot', 
-                           'q2d', 'q_in_band_form', 'electron_phonon', 'el_ph_nsigma', 
-                           'el_ph_sigma', 'ahc_dir', 'ahc_nbnd', 'ahc_nbndskip', 
-                           'skip_upperfan', 'lshift_q', 'zeu', 'zue', 'elop', 'fpol', 
-                           'ldisp', 'nogg', 'asr', 'ldiag', 'lqdir', 'search_sym', 
-                           'nq1', 'nq2', 'nq3', 'nk1', 'nk2', 'nk3', 'k1', 'k2', 'k3', 
-                           'diagonalization', 'read_dns_bare', 'ldvscf_interpolate', 
-                           'wpot_dir', 'do_long_range', 'do_charge_neutral', 'start_irr', 
-                           'last_irr', 'nat_todo', 'modenum', 'start_q', 'last_q', 
-                           'dvscf_star', 'drho_star', ],
-                'LINE': ['xq', 'atom'],
-                # 'qPointsSpecs': ['nqs', 'xq1', 'xq2', 'xq3', 'nq'],
-                            },
-        'dynmat': {'INPUT': ['fildyn', 'q', 'amass', 'asr', 'axis', 'lperm', 'lplasma', 'filout', 
-                             'fileig', 'filmol', 'filxsf', 'loto_2d', 'el_ph_nsig', 'el_ph_sigma']
-                  },
-        'matdyn': {'INPUT': ['flfrc', 'asr', 'dos', 'nk1', 'nk2', 'nk3', 'deltaE', 'ndos', 'fldos', 
-                             'flfrq', 'flvec', 'fleig', 'fldvn', 'at', 'l1', 'l2', 'l3', 'ntyp', 
-                             'amass', 'readtau', 'fltau', 'la2F', 'q_in_band_form', 'q_in_cryst_coord', 
-                             'eigen_similarity', 'fd', 'na_ifc', 'nosym', 'loto_2d', 
-                             'loto_disable']
-                  },
-        'q2r': {'INPUT': ['fildyn', 'flfrc', 'zasr', 'loto_2d'],},
-        }
         
-        
-        package_parameters = state_parameters[package]
-        filename = os.path.join(self.directory, '%s.%si' %(self.prefix, package))
-        with open(filename, 'w') as f:
-            for section, parameters in package_parameters.items():
-                if section != 'LINE':
-                    f.write('&%s\n'%section)
-                    for key, value in kwargs.items():
-                        if key in parameters:
-                            if isinstance(value, dict):
-                                for subkey, subvalue in value.items():
-                                    if isinstance(subvalue, str):
-                                        f.write('  %s(%s) = "%s", \n' %(key, subkey, subvalue))
-                                    else:
-                                        f.write('  %s(%s) = %s, \n' %(key, subkey, subvalue))
-                            else:
-                                if isinstance(value, str):
-                                    f.write('  {0:10s} =  "{1}" \n'.format(key, value))
-                                else:
-                                    f.write('  {0:10s} =  {1} \n'.format(key, value))
-                    f.write('/ \n')
-                else:
-                    for key, value in kwargs.items():
-                        if key in parameters:
-                            f.write('  %s \n' %(value))
-
-    def post_calculate(self):
-        command = self.command
-        print('Running %s'%self.package)
-        try:
-            proc = subprocess.Popen(command, shell=True, cwd=self.directory)
-        except OSError as err:
-            msg = 'Failed to execute "{}"'.format(command)
-            raise EnvironmentError(msg) from err
-
-        errorcode = proc.wait()
-
-        if errorcode:
-            path = os.path.abspath(self.directory)
-            msg = ('Command "{}" failed in '
-                   '{} with error code {}'.format(command,
-                                                  path, errorcode))
-            raise CalculationFailed(msg)
-        print('Done: %s' %self.package)
-    def post_read_results(self):
-        '''
-        '''
-        pass
     def plot_phdos(self, fldos = None, ax = None, output = None):
         '''
         '''
