@@ -63,6 +63,7 @@ def write_espresso_in(filename, atoms, input_data={}, pseudopotentials=None,
 def build_section_str(atoms, species_info, input_data, input_parameters):
     '''
     '''
+    from ase.io.espresso import cell_to_ibrav, ibrav_to_cell
     # Add computed parameters
     # different magnetisms means different types
     input_parameters['system']['ntyp'] = len(species_info)
@@ -124,9 +125,8 @@ def build_atomic_species_str(atoms, input_parameters, pseudopotentials):
 
     # Species info holds the information on the pseudopotential and
     # associated for each element
-    if 'species' not in atoms.info:
-        # print('no species')
-        atoms.info['species'] = atoms.get_chemical_symbols()
+    if 'species' not in atoms.arrays:
+        atoms.new_array('species', np.array(atoms.get_chemical_symbols(), dtype = 'U20'))
     if pseudopotentials is None:
         pseudopotentials = {}
     species_info = {}
@@ -134,7 +134,7 @@ def build_atomic_species_str(atoms, input_parameters, pseudopotentials):
     ntyp = 0
     # Convert atoms into species.
     for i in range(len(atoms)):
-        species = atoms.info['species'][i]
+        species = atoms.arrays['species'][i]
         if species not in species_info:
             ntyp += 1
             species_info[species] = {}
@@ -179,6 +179,7 @@ def build_cell_str(atoms, input_parameters):
 def build_kpts_str(atoms, kspacing, kpts, koffset):
     '''
     '''
+    from ase.io.espresso import kspacing_to_grid
     # KPOINTS - add a MP grid as required
     if kspacing is not None:
         kgrid = kspacing_to_grid(atoms, kspacing)
@@ -222,10 +223,10 @@ def build_atomic_positions_str(atoms, crystal_coordinates):
 
     '''
     import sys
-    if 'species' not in atoms.info:
-        atoms.info['species'] = atoms.get_chemical_symbols()
+    if 'species' not in atoms.arrays:
+        atoms.new_array('species', np.array(atoms.get_chemical_symbols(), dtype = 'U20'))
     else:
-        if len(atoms.info['species']) != len(atoms):
+        if len(atoms.arrays['species']) != len(atoms):
             sys.exit('Species is wrong, please check!')
     # Convert ase constraints to QE constraints
     # Nx3 array of force multipliers matches what QE uses
@@ -245,7 +246,7 @@ def build_atomic_positions_str(atoms, crystal_coordinates):
         atomic_positions_str = ['ATOMIC_POSITIONS angstrom\n']
     for i in range(len(atoms)):
         atom = atoms[i]
-        species = atoms.info['species'][i]
+        species = atoms.arrays['species'][i]
         # only inclued mask if something is fixed
         if not all(constraint_mask[atom.index]):
             mask = ' {mask[0]} {mask[1]} {mask[2]}'.format(
